/*
    Melody maid
    copyright (c) 1998-2012 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "mm_dialog.h"
#include "mm_list.h"


enum
{
  ESCAPE_SIGNAL,
  LAST_SIGNAL
};


static gint mm_dialog_signals[LAST_SIGNAL] = {0};


/******************************************************************************
*                                                                             *
******************************************************************************/
G_DEFINE_TYPE (MmDialog, mm_dialog, GTK_TYPE_DIALOG);
static void mm_dialog_escape (MmDialog *dialog);


static void
mm_dialog_class_init (MmDialogClass *klass)
{
  GtkBindingSet *binding_set;

  klass->escape = mm_dialog_escape;

  mm_dialog_signals[ESCAPE_SIGNAL]
                = g_signal_new ("escape",
                                G_TYPE_FROM_CLASS (klass),
                                G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
                                G_STRUCT_OFFSET (MmDialogClass, escape),
                                NULL, NULL,
                                g_cclosure_marshal_VOID__VOID,
                                G_TYPE_NONE, 0);

  binding_set = gtk_binding_set_by_class (klass);
  gtk_binding_entry_add_signal (binding_set, GDK_KEY_Escape, 0, "escape", 0);
}


static void
mm_dialog_escape (MmDialog *dialog)
{
  if (gtk_widget_get_visible (dialog->abort_button)
                        && gtk_widget_get_sensitive (dialog->abort_button))
    g_signal_emit_by_name (G_OBJECT (dialog->abort_button), "clicked");
  else if (gtk_widget_get_visible (dialog->cancel_button)
                        && gtk_widget_get_sensitive (dialog->cancel_button))
    g_signal_emit_by_name (G_OBJECT (dialog->cancel_button), "clicked");
  else if (gtk_widget_get_visible (dialog->ok_button)
                        && gtk_widget_get_sensitive (dialog->ok_button))
    g_signal_emit_by_name (G_OBJECT (dialog->ok_button), "clicked");
}


/* ja:閉じるボタンが押された */
static gboolean
mm_dialog_delete (GtkWidget *widget,
                  GdkEvent  *event,
                  gpointer   user_data)
{
  if (gtk_widget_get_visible (MM_DIALOG (widget)->abort_button)
            && gtk_widget_get_sensitive (MM_DIALOG (widget)->abort_button))
    g_signal_emit_by_name
                    (G_OBJECT (MM_DIALOG (widget)->abort_button), "clicked");
  else if (gtk_widget_get_visible (MM_DIALOG (widget)->cancel_button)
            && gtk_widget_get_sensitive (MM_DIALOG (widget)->cancel_button))
    g_signal_emit_by_name
                    (G_OBJECT (MM_DIALOG (widget)->cancel_button), "clicked");
  else if (gtk_widget_get_visible (MM_DIALOG (widget)->ok_button)
            && gtk_widget_get_sensitive (MM_DIALOG (widget)->ok_button))
    g_signal_emit_by_name
                    (G_OBJECT (MM_DIALOG (widget)->ok_button), "clicked");
  else
    return FALSE;
  return TRUE;
}


static void
mm_dialog_init (MmDialog *dialog)
{
  GtkWidget *scroll;

  /* ja:メインウインドウ */
  gtk_window_set_default_size (GTK_WINDOW (dialog),
                    gdk_screen_width () * 2 / 3, gdk_screen_height () * 2 / 3);
  g_signal_connect (G_OBJECT (dialog), "delete-event",
                                        G_CALLBACK (mm_dialog_delete), NULL);
  /* ja:ボタン */
  dialog->abort_button = gtk_dialog_add_button (GTK_DIALOG (dialog),
                                        GTK_STOCK_STOP, GTK_RESPONSE_REJECT);
#ifdef G_OS_WIN32
  dialog->ok_button = gtk_dialog_add_button (GTK_DIALOG (dialog),
                                        GTK_STOCK_OK, GTK_RESPONSE_OK);
  dialog->cancel_button = gtk_dialog_add_button (GTK_DIALOG (dialog),
                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
#else /* not G_OS_WIN32 */
  dialog->cancel_button = gtk_dialog_add_button (GTK_DIALOG (dialog),
                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
  dialog->ok_button = gtk_dialog_add_button (GTK_DIALOG (dialog),
                                        GTK_STOCK_OK, GTK_RESPONSE_OK);
#endif /* not G_OS_WIN32 */
  /* ja:リスト */
  dialog->list = mm_list_new ();
  /* ja:スクロールウインドウ */
  scroll = gtk_scrolled_window_new (NULL, NULL);
  gtk_container_add (GTK_CONTAINER (scroll), dialog->list);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
                                GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_container_set_border_width (GTK_CONTAINER (scroll), SPACING);
  /* ja:フレームとボックス */
  gtk_box_pack_end (GTK_BOX (gtk_dialog_get_content_area
                                (GTK_DIALOG (dialog))), scroll, TRUE, TRUE, 0);

  /* ja:表示 */
  gtk_widget_show_all (scroll);
  gtk_widget_hide (dialog->abort_button);
  gtk_widget_hide (dialog->ok_button);
  gtk_widget_hide (dialog->cancel_button);
}


/******************************************************************************
*                                                                             *
* ja:更新関数群                                                               *
*                                                                             *
******************************************************************************/
/*  ja:新規作成
    RET,ウィジェット                                                        */
GtkWidget *
mm_dialog_new (void)
{
  return GTK_WIDGET (g_object_new (MM_TYPE_DIALOG, NULL));
}
